package com.ibm.oti.shared;

import java.security.AccessController;
import com.ibm.oti.util.PriviAction;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

/**
 * Global class which provides SharedClassHelperFactory, SharedDataHelperFactory and sharing status.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassHelperFactory
 * @see SharedDataHelperFactory
 */
public class Shared {

	private static final String ENABLED_PROPERTY_NAME = "com.ibm.oti.shared.enabled";
	private static Object monitor;
	private static SharedClassHelperFactory shcHelperFactory;
	private static SharedDataHelperFactory shdHelperFactory;

	private static class Monitor {}

	static {
		monitor = new Monitor();
	}

	private static String getSharedClassEnabled() {
		return (String)AccessController.doPrivileged(new PriviAction(ENABLED_PROPERTY_NAME));
	}

	/**
	 * Returns true if sharing is enabled for this JVM (using -Xshareclasses on the command-line)
	 * <p>
	 * @author		OTI
	 * @version		initial
	 */
	public static boolean isSharingEnabled() {
		String enabled = getSharedClassEnabled();

		return (enabled.equalsIgnoreCase("true"));
	}

	/**
	 * If sharing is enabled returns a SharedClassHelperFactory, otherwise returns null.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		SharedClassHelperFactory
	 */
	public static SharedClassHelperFactory getSharedClassHelperFactory() {
		synchronized(monitor) {
			if (shcHelperFactory==null && isSharingEnabled()) {
				shcHelperFactory = new SharedClassHelperFactoryImpl();
			}
			return shcHelperFactory;
		}
	}

	/**
	 * If sharing is enabled returns a SharedDataHelperFactory, otherwise returns null.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		SharedDataHelperFactory
	 */
	public static SharedDataHelperFactory getSharedDataHelperFactory() {
		synchronized(monitor) {
			if (shdHelperFactory==null && isSharingEnabled()) {
				shdHelperFactory = new SharedDataHelperFactoryImpl();
			}
			return shdHelperFactory;
		}
	}
}
